@@ -18,22 +18,19 @@
  */
 package org.apache.myfaces.context.servlet;
 
-import java.io.IOException;
-import java.util.ArrayList;
-import java.util.Collection;
-import java.util.Collections;
-import java.util.EnumSet;
-import java.util.List;
-import java.util.Set;
-import java.util.logging.Level;
-import java.util.logging.Logger;
+import org.apache.myfaces.application.ResourceHandlerImpl;
+import org.apache.myfaces.context.PartialResponseWriterImpl;
+import org.apache.myfaces.context.RequestViewContext;
+import org.apache.myfaces.renderkit.html.HtmlResponseStateManager;
+import org.apache.myfaces.shared.renderkit.JSFAttr;
+import org.apache.myfaces.shared.util.StringUtils;
 
 import javax.faces.FactoryFinder;
+import javax.faces.component.EditableValueHolder;
 import javax.faces.component.UIComponent;
 import javax.faces.component.UIViewParameter;
 import javax.faces.component.UIViewRoot;
-import javax.faces.component.html.HtmlBody;
-import javax.faces.component.html.HtmlHead;
+import javax.faces.component.behavior.ClientBehaviorContext;
 import javax.faces.component.visit.VisitCallback;
 import javax.faces.component.visit.VisitContext;
 import javax.faces.component.visit.VisitContextFactory;
@@ -45,23 +42,29 @@ import javax.faces.context.PartialResponseWriter;
 import javax.faces.context.PartialViewContext;
 import javax.faces.context.ResponseWriter;
 import javax.faces.event.PhaseId;
+import javax.faces.lifecycle.ClientWindow;
 import javax.faces.render.RenderKit;
 import javax.faces.render.RenderKitFactory;
 import javax.faces.view.ViewMetadata;
-
-import org.apache.myfaces.context.PartialResponseWriterImpl;
-import org.apache.myfaces.context.RequestViewContext;
-import org.apache.myfaces.shared.config.MyfacesConfig;
-import org.apache.myfaces.shared.util.ExternalContextUtils;
-import org.apache.myfaces.shared.util.StringUtils;
+import java.io.IOException;
+import java.util.ArrayList;
+import java.util.Collection;
+import java.util.Collections;
+import java.util.EnumSet;
+import java.util.List;
+import java.util.Map;
+import java.util.Set;
+import java.util.logging.Level;
+import java.util.logging.Logger;
 
 public class PartialViewContextImpl extends PartialViewContext
 {
 
     private static final String FACES_REQUEST = "Faces-Request";
     private static final String PARTIAL_AJAX = "partial/ajax";
+    private static final String PARTIAL_AJAX_REQ = "javax.faces.partial.ajax";
     private static final String PARTIAL_PROCESS = "partial/process";
-    private static final String SOURCE_PARAM_NAME = "javax.faces.source";
+
     /**
      * Internal extension for
      * https://issues.apache.org/jira/browse/MYFACES-2841
@@ -76,6 +79,11 @@ public class PartialViewContextImpl extends PartialViewContext
     private static final  Set<VisitHint> PARTIAL_RENDER_HINTS = 
             Collections.unmodifiableSet(EnumSet.of(VisitHint.SKIP_UNRENDERED));
 
+    private static final VisitCallback RESET_VALUES_CALLBACK = new ResetValuesCallback();
+
+    private static final  Set<VisitHint> RESET_VALUES_HINTS =
+            Collections.unmodifiableSet(EnumSet.of(VisitHint.SKIP_UNRENDERED));
+
     private FacesContext _facesContext = null;
     private boolean _released = false;
     // Cached values, since their parent methods could be called
@@ -97,6 +105,8 @@ public class PartialViewContextImpl extends PartialViewContext
     private Boolean _renderAll = null;
     private PartialResponseWriter _partialResponseWriter = null;
     private VisitContextFactory _visitContextFactory = null;
+    private Boolean _resetValues = null;
+    private List<String> _evalScripts = new ArrayList<String>();
 
     public PartialViewContextImpl(FacesContext context)
     {
@@ -114,21 +124,18 @@ public class PartialViewContextImpl extends PartialViewContext
     public boolean isAjaxRequest()
     {
         assertNotReleased();
-        /*
-         * Internal extension for
-         * https://issues.apache.org/jira/browse/MYFACES-2841
-         * will be changed for 2.1 to the official marker
-         */
-        if (_iframeRequest == null) {
-            isIFrameRequest();
-        }
-        if (_ajaxRequest == null) {
+        if (_ajaxRequest == null)
+        {
             String requestType = _facesContext.getExternalContext().
-                    getRequestHeaderMap().get(FACES_REQUEST);
+                   getRequestHeaderMap().get(FACES_REQUEST);
             _ajaxRequest = (requestType != null && PARTIAL_AJAX.equals(requestType));
+            String reqParmamterPartialAjax = _facesContext.getExternalContext().
+                    getRequestParameterMap().get(PARTIAL_AJAX_REQ);
+            //jsdoc reference in an ajax request the javax.faces.partial.ajax must be set as ajax parameter
+            //the other one is Faces-Request == partial/ajax which is basically the same
+            _ajaxRequest = _ajaxRequest || reqParmamterPartialAjax != null;
         }
-        //for now we have to treat the partial iframe request also as ajax request
-        return _ajaxRequest || _iframeRequest;
+        return _ajaxRequest;
     }
 
     @Override
@@ -255,7 +262,7 @@ public class PartialViewContextImpl extends PartialViewContext
                 // with, e.g., a button).
 
                 String source = _facesContext.getExternalContext().getRequestParameterMap().get
-                        (PartialViewContextImpl.SOURCE_PARAM_NAME);
+                        (ClientBehaviorContext.BEHAVIOR_SOURCE_PARAM_NAME);
 
                 if (source != null)
                 {
@@ -388,6 +395,12 @@ public class PartialViewContextImpl extends PartialViewContext
         return _partialResponseWriter;
     }
 
+    @Override
+    public List<String> getEvalScripts()
+    {
+        return _evalScripts;
+    }
+
     /**
      * process the partial response
      * allowed phase ids according to the spec
@@ -462,9 +475,21 @@ public class PartialViewContextImpl extends PartialViewContext
 
         try
         {
+            String currentEncoding = writer.getCharacterEncoding();
+
             writer.startDocument();
+            
+            writer.writeAttribute("id", viewRoot.getContainerClientId(_facesContext),"id");
+            
             inDocument = true;
             _facesContext.setResponseWriter(writer);
+            
+            if (isResetValues())
+            {
+                VisitContext visitContext = VisitContext.createVisitContext(_facesContext, getRenderIds(),
+                        RESET_VALUES_HINTS);
+                viewRoot.visitTree(visitContext, RESET_VALUES_CALLBACK);
+            }
 
             if (pvc.isRenderAll())
             {
@@ -483,42 +508,16 @@ public class PartialViewContextImpl extends PartialViewContext
                     }
                     else
                     {
-                        List<UIComponent> updatedComponents = null;
-                        if (!ExternalContextUtils.isPortlet(_facesContext.getExternalContext()) &&
-                                MyfacesConfig.getCurrentInstance(externalContext).isStrictJsf2RefreshTargetAjax())
-                        {
-                            RequestViewContext rvc = RequestViewContext.getCurrentInstance(_facesContext);
-                            if (rvc.isRenderTarget("head"))
-                            {
-                                UIComponent head = findHeadComponent(viewRoot);
-                                if (head != null)
-                                {
-                                    writer.startUpdate("javax.faces.ViewHead");
-                                    head.encodeAll(_facesContext);
-                                    writer.endUpdate();
-                                    if (updatedComponents == null)
-                                    {
-                                        updatedComponents = new ArrayList<UIComponent>();
-                                    }
-                                    updatedComponents.add(head);
-                                }
-                            }
-                            if (rvc.isRenderTarget("body") || rvc.isRenderTarget("form"))
-                            {
-                                UIComponent body = findBodyComponent(viewRoot);
-                                if (body != null)
-                                {
-                                    writer.startUpdate("javax.faces.ViewBody");
-                                    body.encodeAll(_facesContext);
-                                    writer.endUpdate();
-                                    if (updatedComponents == null)
-                                    {
-                                        updatedComponents = new ArrayList<UIComponent>();
-                                    }
-                                    updatedComponents.add(body);
-                                }
-                            }
-                        }
+                        // In JSF 2.3 it was added javax.faces.Resource as an update target to add scripts or
+                        // stylesheets inside <head> tag. In that sense 
+                        // org.apache.myfaces.STRICT_JSF_2_REFRESH_TARGET_AJAX web config param, which was a 
+                        // workaround for dynamic refresh can be deprecated.
+                        
+                        List<UIComponent> updatedComponents = new ArrayList<UIComponent>();
+                        RequestViewContext rvc = RequestViewContext.getCurrentInstance(_facesContext);
+                        processRenderResource(_facesContext, writer, rvc, updatedComponents, "head");
+                        processRenderResource(_facesContext, writer, rvc, updatedComponents, "body");
+                        processRenderResource(_facesContext, writer, rvc, updatedComponents, "form");
 
                         VisitContext visitCtx = getVisitContextFactory().getVisitContext(
                                 _facesContext, renderIds, PARTIAL_RENDER_HINTS);
@@ -526,29 +525,23 @@ public class PartialViewContextImpl extends PartialViewContext
                                            new PhaseAwareVisitCallback(_facesContext, phaseId, updatedComponents));
                     }
                 }
-                else if (!ExternalContextUtils.isPortlet(_facesContext.getExternalContext()) &&
-                        MyfacesConfig.getCurrentInstance(externalContext).isStrictJsf2RefreshTargetAjax())
+                else
                 {
+                    List<UIComponent> updatedComponents = new ArrayList<UIComponent>();
                     RequestViewContext rvc = RequestViewContext.getCurrentInstance(_facesContext);
-                    if (rvc.isRenderTarget("head"))
+                    processRenderResource(_facesContext, writer, rvc, updatedComponents, "head");
+                    processRenderResource(_facesContext, writer, rvc, updatedComponents, "body");
+                    processRenderResource(_facesContext, writer, rvc, updatedComponents, "form");
+                }
+                
+                List<String> evalScripts = pvc.getEvalScripts();
+                if (evalScripts != null && evalScripts.size() > 0)
+                {
+                    for (String script : evalScripts)
                     {
-                        UIComponent head = findHeadComponent(viewRoot);
-                        if (head != null)
-                        {
-                            writer.startUpdate("javax.faces.ViewHead");
-                            head.encodeAll(_facesContext);
-                            writer.endUpdate();
-                        }
-                    }
-                    if (rvc.isRenderTarget("body") || rvc.isRenderTarget("form"))
-                    {
-                        UIComponent body = findBodyComponent(viewRoot);
-                        if (body != null)
-                        {
-                            writer.startUpdate("javax.faces.ViewBody");
-                            body.encodeAll(_facesContext);
-                            writer.endUpdate();
-                        }
+                        writer.startEval();
+                        writer.write(script);
+                        writer.endEval();
                     }
                 }
             }
@@ -570,10 +563,30 @@ public class PartialViewContextImpl extends PartialViewContext
             String viewState = _facesContext.getApplication().getStateManager().getViewState(_facesContext);
             if (viewState != null)
             {
-                writer.startUpdate(PartialResponseWriter.VIEW_STATE_MARKER);
+                writer.startUpdate(HtmlResponseStateManager.generateUpdateViewStateId(
+                    _facesContext));
                 writer.write(viewState);
                 writer.endUpdate();
             }
+            else if (viewRoot.isTransient())
+            {
+                //TODO: fix javascript side, so the field is not removed on ajax form update
+                writer.startUpdate(HtmlResponseStateManager.generateUpdateViewStateId(
+                    _facesContext));
+                writer.write("stateless");
+                writer.endUpdate();
+                //END TODO
+            }
+            
+            
+            ClientWindow cw = _facesContext.getExternalContext().getClientWindow();
+            if (cw != null)
+            {
+                writer.startUpdate(HtmlResponseStateManager.generateUpdateClientWindowId(
+                    _facesContext));
+                writer.write(cw.getId());
+                writer.endUpdate();
+            }
         }
         catch (IOException ex)
         {
@@ -608,21 +621,95 @@ public class PartialViewContextImpl extends PartialViewContext
 
     }
 
+    // same like UIViewRoot#ResetValuesCallback
+    private static class ResetValuesCallback implements VisitCallback
+    {
+        public VisitResult visit(VisitContext context, UIComponent target)
+        {
+            if (target instanceof EditableValueHolder)
+            {
+                ((EditableValueHolder)target).resetValue();
+            }
+            return VisitResult.ACCEPT;
+        }
+    }
+
+    private void processRenderResource(FacesContext facesContext, PartialResponseWriter writer, RequestViewContext rvc, 
+            List<UIComponent> updatedComponents, String target) throws IOException
+    {
+        if (rvc.isRenderTarget(target))
+        {
+            List<UIComponent> list = rvc.getRenderTargetComponentList(target);
+            if (list != null && !list.isEmpty())
+            {
+                writer.startUpdate("javax.faces.Resource");
+                for (UIComponent component : list)
+                {
+                    boolean resourceRendered = false;
+                    if ("javax.faces.resource.Script".equals(component.getRendererType()) ||
+                        "javax.faces.resource.Stylesheet".equals(component.getRendererType()))
+                    {
+                        String resourceName = (String) 
+                                component.getAttributes().get(JSFAttr.NAME_ATTR);
+                        String libraryName = (String) 
+                                component.getAttributes().get(JSFAttr.LIBRARY_ATTR);
+
+                        if (resourceName == null)
+                        {
+                            // No resource, render all
+                            component.encodeAll(facesContext);
+                            continue;
+                        }
+                        if ("".equals(resourceName))
+                        {
+                            // No resource, render all
+                            component.encodeAll(facesContext);
+                            continue;
+                        }
+
+                        int index = resourceName.indexOf('?');
+                        if (index >= 0)
+                        {
+                            resourceName = resourceName.substring(0, index);
+                        }
+                        // Is resource, render only if it has not been rendered before.
+                        if (!_facesContext.getApplication().getResourceHandler().isResourceRendered(
+                                _facesContext, resourceName, libraryName))
+                        {
+                            component.encodeAll(facesContext);
+                        }
+                    }
+                    else
+                    {
+                        component.encodeAll(facesContext);
+                    }
+                    if (!resourceRendered)
+                    {
+                        if (updatedComponents == null)
+                        {
+                            updatedComponents = new ArrayList<UIComponent>();
+                        }
+                        updatedComponents.add(component);
+                    }
+                }
+                writer.endUpdate();
+            }
+        }
+    }
+
     private void processRenderAll(UIViewRoot viewRoot, PartialResponseWriter writer) throws IOException
     {
-        //java.util.Iterator<UIComponent> iter = viewRoot.getFacetsAndChildren();
+        // Before render all we need to clear rendered resources set to be sure every component resource is
+        // rendered. Remember renderAll means the whole page is replaced, so everything inside <head> is replaced.
+        // and there is no way to diff between the old and the new content of <head>.
+        Map<String, Boolean> map = (Map) viewRoot.getTransientStateHelper().getTransient(
+                ResourceHandlerImpl.RENDERED_RESOURCES_SET);
+        if (map != null)
+        {
+            map.clear();
+        }
+        
         writer.startUpdate(PartialResponseWriter.RENDER_ALL_MARKER);
-        //while (iter.hasNext()) 
-        //{ 
-        //UIComponent comp = iter.next();
-
-        //TODO: Do not check for a specific instance,
-        //just render all children.
-        //if (comp instanceof javax.faces.component.html.HtmlBody)
-        //{
-        //comp.encodeAll (_facesContext);
-        //}
-        //}
         for (int i = 0, childCount = viewRoot.getChildCount(); i < childCount; i++)
         {
             UIComponent comp = viewRoot.getChildren().get(i);
@@ -655,50 +742,6 @@ public class PartialViewContextImpl extends PartialViewContext
         _facesContext = null;
         _released = true;
     }
-
-    private UIComponent findHeadComponent(UIViewRoot root)
-    {
-        for (UIComponent child : root.getChildren())
-        {
-            if (child instanceof HtmlHead)
-            {
-                return child;
-            }
-            else
-            {
-                for (UIComponent grandchild : child.getChildren())
-                {
-                    if (grandchild instanceof HtmlHead)
-                    {
-                        return grandchild;
-                    }
-                }
-            }
-        }
-        return null;
-    }
-
-    private UIComponent findBodyComponent(UIViewRoot root)
-    {
-        for (UIComponent child : root.getChildren())
-        {
-            if (child instanceof HtmlBody)
-            {
-                return child;
-            }
-            else
-            {
-                for (UIComponent grandchild : child.getChildren())
-                {
-                    if (grandchild instanceof HtmlBody)
-                    {
-                        return grandchild;
-                    }
-                }
-            }
-        }
-        return null;
-    }
     
     private VisitContextFactory getVisitContextFactory()
     {
@@ -709,6 +752,18 @@ public class PartialViewContextImpl extends PartialViewContext
         return _visitContextFactory;
     }
 
+    @Override
+    public boolean isResetValues()
+    {
+        if (_resetValues == null)
+        {
+            String value = _facesContext.getExternalContext().getRequestParameterMap().
+                get(RESET_VALUES_PARAM_NAME);
+            _resetValues = "true".equals(value);
+        }
+        return _resetValues;
+    }
+
     private class PhaseAwareVisitCallback implements VisitCallback
     {
 
@@ -731,6 +786,7 @@ public class PartialViewContextImpl extends PartialViewContext
             this._alreadyUpdatedComponents = alreadyUpdatedComponents;
         }
 
+        @Override
         public VisitResult visit(VisitContext context, UIComponent target)
         {
             if (_phaseId == PhaseId.APPLY_REQUEST_VALUES)
