@@ -48,25 +48,33 @@ public class HtmlTextRendererBase
     private static final Logger log = Logger.getLogger(HtmlTextRendererBase.class.getName());
 
     private static final String AUTOCOMPLETE_VALUE_OFF = "off";
+    
+    @Override
+    public void encodeBegin(FacesContext facesContext, UIComponent component) throws IOException
+    {
+      org.apache.myfaces.shared.renderkit.RendererUtils.checkParamValidity(facesContext,component,null);
+      
+      Map<String, List<ClientBehavior>> behaviors = null;
+      if (component instanceof ClientBehaviorHolder)
+      {
+          behaviors = ((ClientBehaviorHolder) component).getClientBehaviors();
+          if (!behaviors.isEmpty())
+          {
+              ResourceUtils.renderDefaultJsfJsInlineIfNecessary(facesContext, facesContext.getResponseWriter());
+          }
+      }
+      if (component instanceof UIInput)
+      {
+          renderInputBegin(facesContext, component);
+      }
+    }
 
     public void encodeEnd(FacesContext facesContext, UIComponent component)
         throws IOException
     {
-        org.apache.myfaces.shared.renderkit.RendererUtils.checkParamValidity(facesContext,component,null);
-        
-        Map<String, List<ClientBehavior>> behaviors = null;
-        if (component instanceof ClientBehaviorHolder)
-        {
-            behaviors = ((ClientBehaviorHolder) component).getClientBehaviors();
-            if (!behaviors.isEmpty())
-            {
-                ResourceUtils.renderDefaultJsfJsInlineIfNecessary(facesContext, facesContext.getResponseWriter());
-            }
-        }
-        
         if (component instanceof UIInput)
         {
-            renderInput(facesContext, component);
+            renderInputEnd(facesContext, component);
         }
         else if (component instanceof UIOutput)
         {
@@ -173,12 +181,10 @@ public class HtmlTextRendererBase
         return true;
     }
 
+    @Deprecated
     protected void renderInput(FacesContext facesContext, UIComponent component)
         throws IOException
     {
-        //allow subclasses to render custom attributes by separating rendering begin and end 
-        renderInputBegin(facesContext, component);
-        renderInputEnd(facesContext, component);
     }
 
     //Subclasses can set the value of an attribute before, or can render a custom attribute after calling this method
@@ -263,7 +269,7 @@ public class HtmlTextRendererBase
 
         if (isDisabled(facesContext, component))
         {
-            writer.writeAttribute(HTML.DISABLED_ATTR, Boolean.TRUE, null);
+            writer.writeAttribute(HTML.DISABLED_ATTR, HTML.DISABLED_ATTR, null);
         }
 
         if (isAutocompleteOff(facesContext, component))
@@ -324,6 +330,24 @@ public class HtmlTextRendererBase
         return false;
     }
 
+    protected boolean isValidLength(FacesContext facesContext, UIInput input)
+    {
+        if (input instanceof HtmlInputText)
+        {
+            int maxlength = ((HtmlInputText)input).getMaxlength();
+            if (maxlength >= 0)
+            {
+                String clientId = input.getClientId(facesContext);
+                String value = facesContext.getExternalContext().getRequestParameterMap().get(clientId);
+                if (value != null && value.length() > maxlength)
+                {
+                    return false;
+                }
+            }
+            
+        }
+        return true;
+    }
 
     public void decode(FacesContext facesContext, UIComponent component)
     {
@@ -331,6 +355,10 @@ public class HtmlTextRendererBase
 
         if (component instanceof UIInput)
         {
+            if (!isValidLength(facesContext, (UIInput)component))
+            {
+                return;
+            }
             HtmlRendererUtils.decodeUIInput(facesContext, component);
             if (component instanceof ClientBehaviorHolder &&
                     !HtmlRendererUtils.isDisabled(component))
