@@ -37,7 +37,6 @@ import javax.faces.component.EditableValueHolder;
 import javax.faces.component.NamingContainer;
 import javax.faces.component.UIComponent;
 import javax.faces.component.UIComponentBase;
-import javax.faces.component.UINamingContainer;
 import javax.faces.component.visit.VisitCallback;
 import javax.faces.component.visit.VisitContext;
 import javax.faces.component.visit.VisitHint;
@@ -48,7 +47,9 @@ import javax.faces.event.FacesEvent;
 import javax.faces.event.FacesListener;
 import javax.faces.event.PhaseId;
 import javax.faces.model.ArrayDataModel;
+import javax.faces.model.CollectionDataModel;
 import javax.faces.model.DataModel;
+import javax.faces.model.IterableDataModel;
 import javax.faces.model.ListDataModel;
 import javax.faces.model.ResultSetDataModel;
 import javax.faces.model.ScalarDataModel;
@@ -56,6 +57,9 @@ import javax.faces.render.Renderer;
 
 import org.apache.myfaces.buildtools.maven2.plugin.builder.annotation.JSFComponent;
 import org.apache.myfaces.buildtools.maven2.plugin.builder.annotation.JSFProperty;
+import org.apache.myfaces.cdi.model.DataModelBuilderProxy;
+import org.apache.myfaces.shared.renderkit.html.util.SharedStringBuilder;
+import org.apache.myfaces.util.ExternalSpecifications;
 
 /**
  *  
@@ -67,7 +71,8 @@ public class UIRepeat extends UIComponentBase implements NamingContainer
 
     public static final String COMPONENT_FAMILY = "facelets";
     
-    //private static final String SKIP_ITERATION_HINT = "javax.faces.visit.SKIP_ITERATION";
+    private static final String STRING_BUILDER_KEY
+            = UIRepeat.class.getName() + ".SHARED_STRING_BUILDER";
 
     private final static DataModel<?> EMPTY_MODEL = new ListDataModel<Object>(Collections.emptyList());
     
@@ -91,6 +96,8 @@ public class UIRepeat extends UIComponentBase implements NamingContainer
     
     // will be set to false if the data should not be refreshed at the beginning of the encode phase
     private boolean _isValidChilds = true;
+    
+    private boolean _emptyModel = false;
 
     private int _end = -1;
     
@@ -98,17 +105,21 @@ public class UIRepeat extends UIComponentBase implements NamingContainer
     
     private int _index = -1;
 
-    private transient StringBuilder _clientIdBuffer;
     private transient Object _origValue;
     private transient Object _origVarStatus;
 
     private transient FacesContext _facesContext;
     
+    static final Integer RESET_MODE_OFF = 0;
+    static final Integer RESET_MODE_SOFT = 1;
+    static final Integer RESET_MODE_HARD = 2;    
+    
     public UIRepeat()
     {
         setRendererType("facelets.ui.Repeat");
     }
 
+    @Override
     public String getFamily()
     {
         return COMPONENT_FAMILY;
@@ -147,6 +158,28 @@ public class UIRepeat extends UIComponentBase implements NamingContainer
         getStateHelper().put(PropertyKeys.step, step );
     }
     
+    @JSFProperty
+    public int getBegin()
+    {
+        return (Integer) getStateHelper().eval(PropertyKeys.begin, -1);
+    }
+
+    public void setBegin(int begin)
+    {
+        getStateHelper().put(PropertyKeys.begin, begin );
+    }
+    
+    @JSFProperty
+    public int getEnd()
+    {
+        return (Integer) getStateHelper().eval(PropertyKeys.end, -1);
+    }
+
+    public void setEnd(int end)
+    {
+        getStateHelper().put(PropertyKeys.end, end );
+    }
+    
     @JSFProperty(literalOnly=true)
     public String getVar()
     {
@@ -200,21 +233,49 @@ public class UIRepeat extends UIComponentBase implements NamingContainer
         {
             return (DataModel) value;
         }
-        else if (value instanceof List)
-        {
-            return new ListDataModel((List<?>) value);
-        }
-        else if (OBJECT_ARRAY_CLASS.isAssignableFrom(value.getClass()))
-        {
-            return new ArrayDataModel((Object[]) value);
-        }
-        else if (value instanceof ResultSet)
-        {
-            return new ResultSetDataModel((ResultSet) value);
-        }
         else
         {
-            return new ScalarDataModel(value);
+            DataModel dataModel = null;
+            if (ExternalSpecifications.isCDIAvailable(getFacesContext().getExternalContext()))
+            {
+                dataModel = (new DataModelBuilderProxy()).createDataModel(
+                        getFacesContext(), value.getClass(), value);
+            }
+            if (dataModel == null)
+            {
+                if (value instanceof List)
+                {
+                    return new ListDataModel((List<?>) value);
+                }
+                else if (OBJECT_ARRAY_CLASS.isAssignableFrom(value.getClass()))
+                {
+                    return new ArrayDataModel((Object[]) value);
+                }
+                else if (value instanceof ResultSet)
+                {
+                    return new ResultSetDataModel((ResultSet) value);
+                }
+                else if (value instanceof Iterable)
+                {
+                    return new IterableDataModel<>((Iterable<?>) value);
+                } 
+                else if (value instanceof Map) 
+                {
+                    return new IterableDataModel<>(((Map<?, ?>) value).entrySet());
+                }
+                else if (value instanceof Collection)
+                {
+                    return new CollectionDataModel((Collection) value);
+                }
+                else
+                {
+                    return new ScalarDataModel(value);
+                }
+            }
+            else
+            {
+                return dataModel;
+            }
         }
     }
     
@@ -262,14 +323,21 @@ public class UIRepeat extends UIComponentBase implements NamingContainer
             return clientId;
         }
 
-        StringBuilder bld = _getBuffer(); //SharedStringBuilder(context);
-        return bld.append(clientId).append(UINamingContainer.getSeparatorChar(context)).append(index).toString();
+        StringBuilder sb = SharedStringBuilder.get(context, STRING_BUILDER_KEY);
+        return sb.append(clientId).append(context.getNamingContainerSeparatorChar()).append(index).toString();
     }
     
     private RepeatStatus _getRepeatStatus()
     {
-        return new RepeatStatus(_count == 0, _index + getStep() >= getDataModel().getRowCount(),
-            _count, _index, getOffset(), _end, getStep());
+        int begin = getBegin();
+        if (begin == -1) 
+        {
+            begin = getOffset();
+        }
+        
+        return new RepeatStatus(_count == 0, _index + getStep() >= getRowCount(),
+            _count, _index, begin, _end, getStep());
+        
     }
 
     private void _captureScopeValues()
@@ -285,22 +353,10 @@ public class UIRepeat extends UIComponentBase implements NamingContainer
             _origVarStatus = getFacesContext().getExternalContext().getRequestMap().get(varStatus);
         }
     }
-    
-    private StringBuilder _getBuffer()
-    {
-        if (_clientIdBuffer == null)
-        {
-            _clientIdBuffer = new StringBuilder();
-        }
-        
-        _clientIdBuffer.setLength(0);
-        
-        return _clientIdBuffer;
-    }
 
     private boolean _isIndexAvailable()
     {
-        return getDataModel().isRowAvailable();
+        return _emptyModel || getDataModel().isRowAvailable();
     }
 
     private void _restoreScopeValues()
@@ -676,7 +732,14 @@ public class UIRepeat extends UIComponentBase implements NamingContainer
      */
     public int getRowCount()
     {
-        return getDataModel().getRowCount();
+        if (_emptyModel) // empty model
+        {
+            return (getEnd() - getBegin())/(getStep() <= 0 ? 1 : getStep());
+        }
+        else 
+        {
+            return getDataModel().getRowCount();
+        }
     }
     
     /**
@@ -687,6 +750,11 @@ public class UIRepeat extends UIComponentBase implements NamingContainer
         return _index;
     }
     
+    public void setRowIndex(int index)
+    {
+        _setIndex(index);
+    }
+    
     private void _setIndex(int index)
     {
         // save child state
@@ -741,10 +809,16 @@ public class UIRepeat extends UIComponentBase implements NamingContainer
         if (_index != -1)
         {
             String var = getVar();
-            if (var != null && localModel.isRowAvailable())
+            if (var != null && (_emptyModel || localModel.isRowAvailable()))
             {
-                getFacesContext().getExternalContext().getRequestMap()
-                        .put(var, localModel.getRowData());
+                if (_emptyModel)
+                {
+                    getFacesContext().getExternalContext().getRequestMap().put(var, _index);
+                }
+                else
+                {
+                    getFacesContext().getExternalContext().getRequestMap().put(var, localModel.getRowData());
+                }
             }
             String varStatus = getVarStatus();
             if (varStatus != null)
@@ -811,45 +885,116 @@ public class UIRepeat extends UIComponentBase implements NamingContainer
 
     private void _validateAttributes() throws FacesException
     {
-        int begin = getOffset();
-        int end = getDataModel().getRowCount();
+        
+        int begin = getBegin();
+        int end = getEnd();
         int size = getSize();
+        _emptyModel = getDataModel() == EMPTY_MODEL && begin != -1 && end != -1;
+        int count = getRowCount();
+        int offset = getOffset();
+        if (begin == -1)
+        {
+            if (size >= 0)
+            {
+                end = getOffset() + getSize();
+            }
+        }      
+       
+        if (end == -1 && size == -1) 
+        {
+            if (begin == -1) 
+            {
+                end = getDataModel().getRowCount();
+            } 
+            else 
+            {
+                end = getDataModel().getRowCount() - 1;
+            }
+        }
+        
         int step = getStep();
-        boolean sizeIsEnd = false;
+        boolean sizeIsEnd = _emptyModel;
+        boolean countdown = _emptyModel && end < begin;
 
         if (size == -1)
         {
-            size = end;
-            sizeIsEnd = true;
+            if (begin == -1)
+            {
+                size =  end;
+                sizeIsEnd = true;
+            } 
+            else 
+            {
+                size = countdown ? (begin - end + 1)/step : (end - begin + 1)/step;
+            }     
         }
+        
+        step = countdown && step > 0 ? -step : step;
 
-        if (end >= 0)
+        if (_emptyModel)
+        {
+            if (step > 0 && (end < begin))
+            {
+                throw new FacesException("on empty models, end cannot be less than begin " +
+                        "when the step is positive");
+            }
+            else if (step < 0 && (end > begin))
+            {
+                throw new FacesException("on empty models, end cannot be greater than begin " +
+                        "when the step is negative");
+            }
+            setStep(step);
+        }
+        else if (end >= 0)
         {
             if (size < 0)
             {
                 throw new FacesException("iteration size cannot be less " +
                         "than zero");
             }
-
-            else if (!sizeIsEnd && (begin + size) > end)
+            else if (!sizeIsEnd && (begin == -1) && (offset + size) > end)
             {
                 throw new FacesException("iteration size cannot be greater " +
                         "than collection size");
             }
+            else if (!sizeIsEnd && (begin == -1) && (offset + size) > count)
+            {
+                throw new FacesException("iteration size cannot be greater " +
+                        "than collection size");
+            }
+            else if (!sizeIsEnd && (begin >= 0) && (begin + size) > end+1)
+            {
+                throw new FacesException("iteration size cannot be greater " +
+                        "than collection size");
+            }
+            else if(!sizeIsEnd && (begin >= 0) && (end+1 > count))
+            {
+                throw new FacesException("end cannot be greater " +
+                        "than collection size");
+            }
+            else if(!sizeIsEnd && (begin >= 0) && (begin > count))
+            {
+                throw new FacesException("begin cannot be greater " +
+                        "than collection size");
+            }
         }
-
-        if ((size > -1) && (begin > end))
+        if (!_emptyModel && (begin >= 0) && (begin > end))
+        {
+            throw new FacesException("begin cannot be greater " +
+                    "than end");
+        }
+        if (!_emptyModel && (size > -1) && (offset > end))
         {
             throw new FacesException("iteration offset cannot be greater " +
                     "than collection size");
         }
 
-        if (step == -1)
+        if (!_emptyModel && step == -1)
         {
             setStep(1);
         }
 
-        if (step < 0)
+        if (!_emptyModel && step < 0)
         {
             throw new FacesException("iteration step size cannot be less " +
                     "than zero");
@@ -861,8 +1006,8 @@ public class UIRepeat extends UIComponentBase implements NamingContainer
                     "to zero");
         }
 
-        _end = size;
-        //_step = step;
+        _end = end;
+        
     }
 
     public void process(FacesContext faces, PhaseId phase)
@@ -885,11 +1030,21 @@ public class UIRepeat extends UIComponentBase implements NamingContainer
             // has children
             if (getChildCount() > 0)
             {
-                int i = getOffset();
-                int end = getSize();
-                int step = getStep();
-                end = (end >= 0) ? i + end : Integer.MAX_VALUE - 1;
+                int i = getOffset();               
                 
+                int begin = getBegin();
+                int end = getEnd(); 
+                if (begin == -1)
+                {
+                    end = getSize();
+                    end = (end >= 0) ? i + end - 1 : Integer.MAX_VALUE - 1;
+                }
+                
+                if (begin >= 0) 
+                {
+                    i = begin;
+                }
+                int step = getStep();
                 // grab renderer
                 String rendererType = getRendererType();
                 Renderer renderer = null;
@@ -901,7 +1056,8 @@ public class UIRepeat extends UIComponentBase implements NamingContainer
                 _count = 0;
                 
                 _setIndex(i);
-                while (i < end && _isIndexAvailable())
+                
+                while (((step > 0 && i <= end) || (step < 0 && i >= end)) && _isIndexAvailable())
                 {
 
                     if (PhaseId.RENDER_RESPONSE.equals(phase) && renderer != null)
@@ -1006,7 +1162,7 @@ public class UIRepeat extends UIComponentBase implements NamingContainer
             {
                 // Check if the clientId for the component, which we 
                 // are looking for, has a rowIndex attached
-                char separator = UINamingContainer.getSeparatorChar(context);
+                char separator = context.getNamingContainerSeparatorChar();
                 String subId = clientId.substring(baseClientId.length() + 1);
                 //If the char next to baseClientId is the separator one and
                 //the subId matches the regular expression
@@ -1106,106 +1262,110 @@ public class UIRepeat extends UIComponentBase implements NamingContainer
     {
         // override the behavior from UIComponent to visit
         // all children once per "row"
-        
-        //(Boolean) context.getFacesContext().getAttributes().get(SKIP_ITERATION_HINT);
-        Boolean skipIterationHint = context.getHints().contains(VisitHint.SKIP_ITERATION);
-        if (skipIterationHint != null && skipIterationHint.booleanValue())
+
+        boolean skipIterationHint = context.getHints().contains(VisitHint.SKIP_ITERATION);
+        if (skipIterationHint)
         {
             return super.visitTree(context, callback);
         }
-        
-        if (!isVisitable(context)) 
-        {
-            return false;
-        }
-        
-        // save the current index, count aside
-        final int prevIndex = _index;
-        final int prevCount = _count;
-        
-        // validate attributes
-        _validateAttributes();
-        
-        // reset index and save scope values
-        _captureScopeValues();
-        _setIndex(-1);
-        
         // push the Component to EL
         pushComponentToEL(context.getFacesContext(), this);
-        try 
+        try
         {
-            VisitResult res = context.invokeVisitCallback(this, callback);
-            switch (res) 
+            if (!isVisitable(context))
             {
-            // we are done, nothing has to be processed anymore
-            case COMPLETE:
-                return true;
-
-            case REJECT:
                 return false;
+            }
 
-            //accept
-            default:
-                // determine if we need to visit our children
-                // Note that we need to do this check because we are a NamingContainer
-                Collection<String> subtreeIdsToVisit = context
-                        .getSubtreeIdsToVisit(this);
-                boolean doVisitChildren = subtreeIdsToVisit != null
-                        && !subtreeIdsToVisit.isEmpty();
-                if (doVisitChildren)
+            // save the current index, count aside
+            final int prevIndex = _index;
+            final int prevCount = _count;
+
+            // reset index and save scope values
+            _captureScopeValues();
+            _setIndex(-1);
+
+            try
+            {
+                VisitResult res = context.invokeVisitCallback(this, callback);
+                switch (res)
                 {
-                    // visit the facets of the component
-                    if (getFacetCount() > 0) 
+                // we are done, nothing has to be processed anymore
+                case COMPLETE:
+                    return true;
+
+                case REJECT:
+                    return false;
+
+                //accept
+                default:
+                    // determine if we need to visit our children
+                    // Note that we need to do this check because we are a NamingContainer
+                    Collection<String> subtreeIdsToVisit = context
+                            .getSubtreeIdsToVisit(this);
+                    boolean doVisitChildren = subtreeIdsToVisit != null
+                            && !subtreeIdsToVisit.isEmpty();
+                    if (doVisitChildren)
                     {
-                        for (UIComponent facet : getFacets().values()) 
+                        // validate attributes
+                        _validateAttributes();
+   
+                        // visit the facets of the component
+                        if (getFacetCount() > 0)
                         {
-                            if (facet.visitTree(context, callback)) 
+                            for (UIComponent facet : getFacets().values())
                             {
-                                return true;
-                            }
-                        }
-                    }
-                    
-                    // visit the children once per "row"
-                    if (getChildCount() > 0) 
-                    {
-                        int i = getOffset();
-                        int end = getSize();
-                        int step = getStep();
-                        end = (end >= 0) ? i + end : Integer.MAX_VALUE - 1;
-                        _count = 0;
-                        
-                        _setIndex(i);
-                        while (i < end && _isIndexAvailable())
-                        {
-                            for (int j = 0, childCount = getChildCount(); j < childCount; j++)
-                            {
-                                UIComponent child = getChildren().get(j);
-                                if (child.visitTree(context, callback)) 
+                                if (facet.visitTree(context, callback))
                                 {
                                     return true;
                                 }
                             }
-                            
-                            _count++;
-                            i += step;
-                            
+                        }
+
+                        // visit the children once per "row"
+                        if (getChildCount() > 0)
+                        {
+                            int i = getOffset();
+                            int end = getSize();
+                            int step = getStep();
+                            end = (end >= 0) ? i + end : Integer.MAX_VALUE - 1;
+                            _count = 0;
+
                             _setIndex(i);
+                            while (i < end && _isIndexAvailable())
+                            {
+                                for (int j = 0, childCount = getChildCount(); j < childCount; j++)
+                                {
+                                    UIComponent child = getChildren().get(j);
+                                    if (child.visitTree(context, callback))
+                                    {
+                                        return true;
+                                    }
+                                }
+
+                                _count++;
+                                i += step;
+
+                                _setIndex(i);
+                            }
                         }
                     }
+                    return false;
                 }
-                return false;
+            }
+            finally
+            {
+
+                // restore the previous count, index and scope values
+                _count = prevCount;
+                _setIndex(prevIndex);
+                _restoreScopeValues();
             }
         }
-        finally 
+        finally
         {
             // pop the component from EL
             popComponentFromEL(context.getFacesContext());
-            
-            // restore the previous count, index and scope values
-            _count = prevCount;
-            _setIndex(prevIndex);
-            _restoreScopeValues();
         }
     }
 
@@ -1368,11 +1528,13 @@ public class UIRepeat extends UIComponentBase implements NamingContainer
             _target.setPhaseId(phaseId);
         }
 
+        @Override
         public boolean isAppropriateListener(FacesListener listener)
         {
             return _target.isAppropriateListener(listener);
         }
 
+        @Override
         public void processListener(FacesListener listener)
         {
             UIRepeat owner = (UIRepeat) getComponent();
@@ -1491,56 +1653,124 @@ public class UIRepeat extends UIComponentBase implements NamingContainer
         super.queueEvent(new IndexedEvent(this, event, _index));
     }
 
-    // -=Leonardo Uribe=- At the moment I haven't found any use case that
-    // require to store the rowStates in the component state, mostly
-    // because EditableValueHolder instances render the value into the
-    // client and then this value are taken back at the beginning of the
-    // next request. So, I just let this code in comments just in case
-    // somebody founds an issue with this.  
-    /* 
-    @SuppressWarnings("unchecked")
     @Override
-    public void restoreState(FacesContext facesContext, Object state)
+    public void restoreState(FacesContext context, Object state)
     {
         if (state == null)
         {
             return;
         }
         
-        Object[] values = (Object[])state;
-        super.restoreState(facesContext,values[0]);
-        if (values[1] == null)
+        Object values[] = (Object[]) state;
+        super.restoreState(context, values[0]);
+        //Object restoredRowStates = UIComponentBase.restoreAttachedState(context, values[1]);
+        /*
+        if (restoredRowStates == null)
         {
-            _rowStates.clear();
+            if (!_rowDeltaStates.isEmpty())
+            {
+                _rowDeltaStates.clear();
+            }
         }
         else
         {
-            _rowStates = (Map<String, Collection<Object[]>>) restoreAttachedState(facesContext, values[1]);
+            _rowDeltaStates = (Map<String, Map<String, Object> >) restoredRowStates;
+        }*/
+        if (values.length > 2)
+        {
+            Object rs = UIComponentBase.restoreAttachedState(context, values[2]);
+            if (rs == null)
+            {
+                if (!_rowStates.isEmpty())
+                {
+                    _rowStates.clear();
+                }
+            }
+            else
+            {
+                _rowStates = (Map<String, Collection<Object[]> >) rs;
+            }
         }
     }
 
     @Override
-    public Object saveState(FacesContext facesContext)
+    public Object saveState(FacesContext context)
     {
+        if (context.getViewRoot() != null)
+        {
+            if (context.getViewRoot().getAttributes().get("oam.view.resetSaveStateMode") == RESET_MODE_SOFT)
+            {
+                _dataModelMap.clear();
+                _isValidChilds=true;
+                //_rowTransientStates.clear();
+            }
+            if (context.getViewRoot().getAttributes().get("oam.view.resetSaveStateMode") == RESET_MODE_HARD)
+            {
+                _dataModelMap.clear();
+                _isValidChilds=true;
+                //_rowTransientStates.clear();
+                _rowStates.clear();
+                //_rowDeltaStates.clear();
+            }
+        }
         if (initialStateMarked())
         {
-            Object parentSaved = super.saveState(facesContext);
-            if (parentSaved == null && _rowStates.isEmpty())
+            Object parentSaved = super.saveState(context);
+            if (context.getCurrentPhaseId() != null && 
+                !PhaseId.RENDER_RESPONSE.equals(context.getCurrentPhaseId()))
             {
-                //No values
-                return null;
-            }   
-            return new Object[]{parentSaved, saveAttachedState(facesContext, _rowStates)};
+                if (parentSaved == null /*&&_rowDeltaStates.isEmpty()*/ && _rowStates.isEmpty())
+                {
+                    return null;
+                }
+                else
+                {
+                    Object values[] = new Object[3];
+                    values[0] = super.saveState(context);
+                    //values[1] = UIComponentBase.saveAttachedState(context, _rowDeltaStates);
+                    values[1] = null;
+                    values[2] = UIComponentBase.saveAttachedState(context, _rowStates);
+                    return values;
+                }
+            }
+            else
+            {
+                if (parentSaved == null /*&&_rowDeltaStates.isEmpty()*/)
+                {
+                    return null;
+                }
+                else
+                {
+                    Object values[] = new Object[2];
+                    values[0] = super.saveState(context);
+                    //values[1] = UIComponentBase.saveAttachedState(context, _rowDeltaStates);
+                    values[1] = null;
+                    return values; 
+                }
+            }
         }
         else
         {
-            Object[] values = new Object[2];
-            values[0] = super.saveState(facesContext);
-            values[1] = saveAttachedState(facesContext, _rowStates);
-            return values;
-        } 
+            if (context.getCurrentPhaseId() != null && 
+                !PhaseId.RENDER_RESPONSE.equals(context.getCurrentPhaseId()))
+            {
+                Object values[] = new Object[3];
+                values[0] = super.saveState(context);
+                //values[1] = UIComponentBase.saveAttachedState(context, _rowDeltaStates);
+                values[1] = null;
+                values[2] = UIComponentBase.saveAttachedState(context, _rowStates);
+                return values; 
+            }
+            else
+            {
+                Object values[] = new Object[2];
+                values[0] = super.saveState(context);
+                //values[1] = UIComponentBase.saveAttachedState(context, _rowDeltaStates);
+                values[1] = null;
+                return values;
+            }
+        }
     }
-    */
     
     @Override
     public void encodeBegin(FacesContext context) throws IOException
@@ -1559,7 +1789,6 @@ public class UIRepeat extends UIComponentBase implements NamingContainer
             // corresponding DataModel element.
             _rowStates.clear();
         }
-        // TODO Auto-generated method stub
         super.encodeBegin(context);
     }
     
@@ -1610,5 +1839,7 @@ public class UIRepeat extends UIComponentBase implements NamingContainer
         , varStatus
         , offset
         , step
+        , begin
+        , end
     }
 }
