@@ -19,6 +19,7 @@
 package javax.faces.component;
 
 import java.io.IOException;
+import java.lang.reflect.Method;
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.Collection;
@@ -36,6 +37,11 @@ import javax.el.MethodExpression;
 import javax.el.ValueExpression;
 import javax.faces.FactoryFinder;
 import javax.faces.application.ProjectStage;
+import javax.faces.application.StateManager;
+import javax.faces.component.visit.VisitCallback;
+import javax.faces.component.visit.VisitContext;
+import javax.faces.component.visit.VisitResult;
+import javax.faces.context.ExternalContext;
 import javax.faces.context.FacesContext;
 import javax.faces.context.PartialViewContext;
 import javax.faces.event.AbortProcessingException;
@@ -58,6 +64,7 @@ import javax.faces.webapp.FacesServlet;
 import org.apache.myfaces.buildtools.maven2.plugin.builder.annotation.JSFComponent;
 import org.apache.myfaces.buildtools.maven2.plugin.builder.annotation.JSFJspProperty;
 import org.apache.myfaces.buildtools.maven2.plugin.builder.annotation.JSFProperty;
+import org.apache.myfaces.buildtools.maven2.plugin.builder.annotation.JSFWebConfigParam;
 
 /**
  * Creates a JSF View, which is a container that holds all of the components that are part of the view.
@@ -78,6 +85,13 @@ public class UIViewRoot extends UIComponentBase implements UniqueIdVendor
     public static final String METADATA_FACET_NAME = "javax_faces_metadata";
     public static final String UNIQUE_ID_PREFIX = "j_id";
     public static final String VIEW_PARAMETERS_KEY = "javax.faces.component.VIEW_PARAMETERS_KEY";
+    
+    /**
+     * @since 2.3
+     */
+    @JSFWebConfigParam(defaultValue="false", expectedValues="true, false", since="2.3")
+    public static final String VIEWROOT_PHASE_LISTENER_QUEUES_EXCEPTIONS_PARAM_NAME
+            = "javax.faces.VIEWROOT_PHASE_LISTENER_QUEUES_EXCEPTIONS";
 
     private transient Logger logger = null;
 
@@ -85,6 +99,46 @@ public class UIViewRoot extends UIComponentBase implements UniqueIdVendor
     private static final PhaseProcessor PROCESS_VALIDATORS_PROCESSOR = new ProcessValidatorPhaseProcessor();
     private static final PhaseProcessor UPDATE_MODEL_PROCESSOR = new UpdateModelPhaseProcessor();
 
+    private static final VisitCallback RESET_VALUES_CALLBACK = new ResetValuesCallback();
+    
+    /**
+     * Class that is used to create the view scope map. This strategy
+     * allows change the implementation of view scope map to use cdi or
+     * whatever without change UIViewRoot implementation.
+     */
+    private static Class<?> VIEW_SCOPE_PROXY_MAP_CLASS = null;
+    
+    private static Class<?> REQUEST_VIEW_CONTEXT_CLASS = null;
+    private static Method REQUEST_VIEW_CONTEXT_GET_INSTANCE = null;
+    private static Method REQUEST_VIEW_CONTEXT_SET_RENDER_TARGET = null;
+
+    static
+    {
+        try
+        {
+            VIEW_SCOPE_PROXY_MAP_CLASS
+                    = _ClassUtils.classForName("org.apache.myfaces.view.ViewScopeProxyMap");
+        }
+        catch (Exception e)
+        {
+            // no op
+        }
+        
+        try
+        {
+            REQUEST_VIEW_CONTEXT_CLASS
+                    = _ClassUtils.classForName("org.apache.myfaces.context.RequestViewContext");
+            REQUEST_VIEW_CONTEXT_GET_INSTANCE = REQUEST_VIEW_CONTEXT_CLASS.getMethod("getCurrentInstance",
+                    new Class[] { FacesContext.class });
+            REQUEST_VIEW_CONTEXT_SET_RENDER_TARGET = REQUEST_VIEW_CONTEXT_CLASS.getMethod("setRenderTarget",
+                    new Class[] { String.class, boolean.class, UIComponent.class });
+        }
+        catch (Exception e)
+        {
+            // no op
+        }
+    }
+
     /**
      * The counter which will ensure a unique component id for every component instance in the tree that doesn't have an
      * id attribute set.
@@ -101,6 +155,7 @@ public class UIViewRoot extends UIComponentBase implements UniqueIdVendor
      * so we can use saveAttachedState and restoreAttachedState methods.
      */
     private Map<String, Object> _viewScope;
+    private transient boolean _restoreViewScopeStateCalled = false;
 
     private transient Lifecycle _lifecycle = null;
     
@@ -116,6 +171,9 @@ public class UIViewRoot extends UIComponentBase implements UniqueIdVendor
     private static final String JAVAX_FACES_LOCATION_BODY = "javax_faces_location_body";
     private static final String JAVAX_FACES_LOCATION_FORM = "javax_faces_location_form";
     
+    private static final String SKIP_VIEW_MAP_SAVE_STATE = "oam.viewPool.SKIP_VIEW_MAP_SAVE_STATE";
+    
+    private transient int _resetSaveStateMode = 0;
     private transient boolean _resourceDependencyUniqueId;
     private transient Map<String,Object> _attributesMap;
     
@@ -197,7 +255,22 @@ public class UIViewRoot extends UIComponentBase implements UniqueIdVendor
                     UIComponent component = it.next();
                     if(componentId.equals(component.getId()) && componentResource != component)
                     {
-                        it.remove();
+                        if (!component.isCachedFacesContext())
+                        {
+                            try
+                            {
+                                component.setCachedFacesContext(context);
+                                it.remove();
+                            }
+                            finally
+                            {
+                                component.setCachedFacesContext(null);
+                            }
+                        }
+                        else
+                        {
+                            it.remove();
+                        }
                     }
                     else if (componentResource == component)
                     {
@@ -213,7 +286,22 @@ public class UIViewRoot extends UIComponentBase implements UniqueIdVendor
                 UIComponent component = it.next();
                 if(componentId.equals(component.getId()) && componentResource != component)
                 {
-                    it.remove();
+                    if (!component.isCachedFacesContext())
+                    {
+                        try
+                        {
+                            component.setCachedFacesContext(context);
+                            it.remove();
+                        }
+                        finally
+                        {
+                            component.setCachedFacesContext(null);
+                        }
+                    }
+                    else
+                    {
+                        it.remove();
+                    }
                 }
                 else if (componentResource == component)
                 {
@@ -225,7 +313,50 @@ public class UIViewRoot extends UIComponentBase implements UniqueIdVendor
         // Add the component resource to the list
         if (!alreadyAdded)
         {
-            componentResources.add(componentResource);
+            if (!componentResource.isCachedFacesContext())
+            {
+                try
+                {
+                    componentResource.setCachedFacesContext(context);
+                    componentResources.add(componentResource);
+                }
+                finally
+                {
+                    componentResource.setCachedFacesContext(context);
+                }
+            }
+            else
+            {
+                componentResources.add(componentResource);
+            }
+
+            // this is required to make dynamic resource loading possible since JSF 2.3
+            if (context.getPartialViewContext().isAjaxRequest())
+            {
+                boolean isBuildingInitialState
+                        = context.getAttributes().containsKey(StateManager.IS_BUILDING_INITIAL_STATE);
+                
+                // FaceletViewDeclarationLanguage.isRefreshingTransientBuild(context)
+                boolean isRefreshTransientBuild
+                        = context.getAttributes().containsKey("org.apache.myfaces.REFRESHING_TRANSIENT_BUILD");
+                
+                boolean isPostAddToViewEventAfterBuildInitialState =
+                        !isBuildingInitialState || (isBuildingInitialState && isRefreshTransientBuild);
+                if (isPostAddToViewEventAfterBuildInitialState)
+                {
+                    try
+                    {
+                        // RequestViewContext requestViewContext = RequestViewContext.getInstance(context);
+                        // requestViewContext.setRenderTarget("head", true, componentResource);
+                        Object requestViewContext = REQUEST_VIEW_CONTEXT_GET_INSTANCE.invoke(null, context);
+                        REQUEST_VIEW_CONTEXT_SET_RENDER_TARGET.invoke(requestViewContext, "head", true, componentResource);
+                    }
+                    catch (Exception e)
+                    {
+                        _getLogger().log(Level.SEVERE, "Could not access RequestViewContext", e);
+                    }
+                }
+            }
         }
     }
 
@@ -340,7 +471,6 @@ public class UIViewRoot extends UIComponentBase implements UniqueIdVendor
      */
     public String createUniqueId(FacesContext context, String seed)
     {
-        StringBuilder bld = _getSharedStringBuilder(context);
 
         // Generate an identifier for a component. The identifier will be prefixed with
         // UNIQUE_ID_PREFIX, and will be unique within this UIViewRoot.
@@ -348,23 +478,43 @@ public class UIViewRoot extends UIComponentBase implements UniqueIdVendor
         {
             if (isResourceDependencyUniqueId())
             {
-                Long uniqueIdCounter = (Long) getStateHelper().get(PropertyKeys.resourceDependencyUniqueIdCounter);
+                Integer uniqueIdCounter = (Integer) getStateHelper().get(
+                    PropertyKeys.resourceDependencyUniqueIdCounter);
                 uniqueIdCounter = (uniqueIdCounter == null) ? 0 : uniqueIdCounter;
-                getStateHelper().put(PropertyKeys.resourceDependencyUniqueIdCounter, (uniqueIdCounter+1L));
-                return bld.append(UNIQUE_ID_PREFIX).append("__rd_").append(uniqueIdCounter).toString();
+                getStateHelper().put(PropertyKeys.resourceDependencyUniqueIdCounter, (uniqueIdCounter+1));
+                if (uniqueIdCounter >= _ComponentUtils.UNIQUE_COMPONENT_RD_IDS_SIZE)
+                {
+                    StringBuilder bld = _getSharedStringBuilder(context);
+                    return bld.append(UNIQUE_ID_PREFIX).append(
+                        _ComponentUtils.RD_ID_PREFIX).append(uniqueIdCounter).toString();
+                }
+                else
+                {
+                    return _ComponentUtils.UNIQUE_COMPONENT_RD_IDS[uniqueIdCounter];
+                }
             }
             else
             {
-                Long uniqueIdCounter = (Long) getStateHelper().get(PropertyKeys.uniqueIdCounter);
+                Integer uniqueIdCounter = (Integer) getStateHelper().get(PropertyKeys.uniqueIdCounter);
                 uniqueIdCounter = (uniqueIdCounter == null) ? 0 : uniqueIdCounter;
-                getStateHelper().put(PropertyKeys.uniqueIdCounter, (uniqueIdCounter+1L));
-                return bld.append(UNIQUE_ID_PREFIX).append("__v_").append(uniqueIdCounter).toString();
+                getStateHelper().put(PropertyKeys.uniqueIdCounter, (uniqueIdCounter+1));
+                if (uniqueIdCounter >= _ComponentUtils.UNIQUE_COMPONENT_V_IDS_SIZE)
+                {
+                    StringBuilder bld = _getSharedStringBuilder(context);
+                    return bld.append(UNIQUE_ID_PREFIX).append(
+                        _ComponentUtils.V_ID_PREFIX).append(uniqueIdCounter).toString();
+                }
+                else
+                {
+                    return _ComponentUtils.UNIQUE_COMPONENT_V_IDS[uniqueIdCounter];
+                }
             }
         }
         // Optionally, a unique seed value can be supplied by component creators which
         // should be included in the generated unique id.
         else
         {
+            StringBuilder bld = _getSharedStringBuilder(context);
             return bld.append(UNIQUE_ID_PREFIX).append(seed).toString();
         }
     }
@@ -523,24 +673,15 @@ public class UIViewRoot extends UIComponentBase implements UniqueIdVendor
         return (MethodExpression) getStateHelper().eval(PropertyKeys.beforePhaseListener);
     }
 
-    /**
-     * DO NOT USE.
-     * <p>
-     * As this component has no "id" property, it has no clientId property either.
-     */
-    @Override
-    public String getClientId(FacesContext context)
-    {
-        return super.getClientId(context);
-        // Call parent method due to TCK problems
-        // return null;
-    }
-
     /**
      * @since 2.0
      */
     public List<UIComponent> getComponentResources(FacesContext context, String target)
     {
+        if (target == null)
+        {
+            throw new NullPointerException("target");
+        }
         // Locate the facet for the component by calling getFacet() using target as the argument
         UIComponent facet = getFacet(target);
 
@@ -590,7 +731,8 @@ public class UIViewRoot extends UIComponentBase implements UniqueIdVendor
         {
             // create the facet by calling context.getApplication().createComponent()
             // using javax.faces.Panel as the argument
-            facet = context.getApplication().createComponent("javax.faces.ComponentResourceContainer");
+            facet = context.getApplication().createComponent(context,
+                "javax.faces.ComponentResourceContainer", null);
 
             // Set the id of the facet to be target
             if (target.equals("head"))
@@ -624,6 +766,22 @@ public class UIViewRoot extends UIComponentBase implements UniqueIdVendor
         }
         return facet.getChildren();
     }
+    
+    /**
+     * @since 2.3
+     * @param context
+     * @return 
+     */
+    public List<UIComponent> getComponentResources(FacesContext context)
+    {
+        List<UIComponent> componentResources = new ArrayList<UIComponent>();
+        
+        componentResources.addAll(_getComponentResources(context, "head"));
+        componentResources.addAll(_getComponentResources(context, "body"));
+        componentResources.addAll(_getComponentResources(context, "form"));
+        
+        return Collections.unmodifiableList(componentResources);
+    }
 
     @Override
     public String getFamily()
@@ -742,9 +900,21 @@ public class UIViewRoot extends UIComponentBase implements UniqueIdVendor
     {
         if (_viewScope == null && create)
         {
-            _viewScope = new ViewScope();
+            if (VIEW_SCOPE_PROXY_MAP_CLASS != null)
+            {
+                _viewScope = (Map<String, Object>)
+                    _ClassUtils.newInstance(VIEW_SCOPE_PROXY_MAP_CLASS);
+            }
+            else
+            {
+                //Default to map for testing purposes
+                _viewScope = new ViewScope();
+            }
             FacesContext facesContext = getFacesContext();
-            facesContext.getApplication().publishEvent(facesContext, PostConstructViewMapEvent.class, this);
+            if (facesContext != null)
+            {
+                facesContext.getApplication().publishEvent(facesContext, PostConstructViewMapEvent.class, UIViewRoot.class, this);
+            }
         }
 
         return _viewScope;
@@ -844,6 +1014,8 @@ public class UIViewRoot extends UIComponentBase implements UniqueIdVendor
                                     boolean beforePhase)
     {
         List<PhaseListener> phaseListeners = (List<PhaseListener>) getStateHelper().get(PropertyKeys.phaseListeners);
+        // Check if any listener was called
+        boolean listenerCalled = false;
         if (listener != null || (phaseListeners != null && !phaseListeners.isEmpty()))
         {
             // how many listeners do we have? (the MethodExpression listener is counted in either way)
@@ -874,6 +1046,7 @@ public class UIViewRoot extends UIComponentBase implements UniqueIdVendor
             // or if the related before PhaseListener finished without an Exception
             if (listener != null && (beforePhase || beforePhaseSuccess[0]))
             {
+                listenerCalled = true;
                 try
                 {
                     listener.invoke(context.getELContext(), new Object[] { event });
@@ -920,6 +1093,7 @@ public class UIViewRoot extends UIComponentBase implements UniqueIdVendor
                         PhaseId listenerPhaseId = phaseListener.getPhaseId();
                         if (phaseId.equals(listenerPhaseId) || PhaseId.ANY_PHASE.equals(listenerPhaseId))
                         {
+                            listenerCalled = true;
                             try
                             {
                                 phaseListener.beforePhase(event);
@@ -931,6 +1105,11 @@ public class UIViewRoot extends UIComponentBase implements UniqueIdVendor
                                 _getLogger().log(Level.SEVERE, "An Exception occured while processing the " +
                                                          "beforePhase method of PhaseListener " + phaseListener +
                                                          " in Phase " + phaseId, t);
+                                if (shouldViewRootPhaseListenerQueuesExceptions(context))
+                                {
+                                    publishException (context, t, phaseId, 
+                                            ExceptionQueuedEventContext.IN_BEFORE_PHASE_KEY);
+                                }
                                 return context.getResponseComplete() ||
                                         (context.getRenderResponse() && !PhaseId.RENDER_RESPONSE.equals(phaseId));
                             }
@@ -959,6 +1138,7 @@ public class UIViewRoot extends UIComponentBase implements UniqueIdVendor
                         if ((phaseId.equals(listenerPhaseId) || PhaseId.ANY_PHASE.equals(listenerPhaseId))
                                 && beforePhaseSuccess[i])
                         {
+                            listenerCalled = true;
                             try
                             {
                                 phaseListener.afterPhase(event);
@@ -968,6 +1148,11 @@ public class UIViewRoot extends UIComponentBase implements UniqueIdVendor
                                 logger.log(Level.SEVERE, "An Exception occured while processing the " +
                                                          "afterPhase method of PhaseListener " + phaseListener +
                                                          " in Phase " + phaseId, t);
+                                if (shouldViewRootPhaseListenerQueuesExceptions(context))
+                                {
+                                    publishException (context, t, phaseId, 
+                                            ExceptionQueuedEventContext.IN_AFTER_PHASE_KEY);
+                                }
                             }
                         }
                     }
@@ -975,14 +1160,43 @@ public class UIViewRoot extends UIComponentBase implements UniqueIdVendor
             }
         }
 
-        if (beforePhase)
+        // The spec javadoc says "... Upon return from the listener, call FacesContext.getResponseComplete() 
+        // and FacesContext.getRenderResponse(). If either return true set the internal state flag to true. ..."
+        // and later it says:
+        // "... Execute any processing for this phase if the internal state flag was not set. ..."
+        // But after some testing it seems if the internal state flag is not set, the check is not done and the
+        // phase is not skipped. The only exception is in render response phase.
+        if (listenerCalled)
         {
-            return context.getResponseComplete() ||
-                    (context.getRenderResponse() && !PhaseId.RENDER_RESPONSE.equals(phaseId));
+            if (beforePhase)
+            {
+                return context.getResponseComplete() ||
+                        (context.getRenderResponse() && !PhaseId.RENDER_RESPONSE.equals(phaseId));
+            }
+            else
+            {
+                return context.getResponseComplete() || context.getRenderResponse();
+            }
         }
         else
         {
-            return context.getResponseComplete() || context.getRenderResponse();
+            if (beforePhase)
+            {
+                if (PhaseId.RENDER_RESPONSE.equals(phaseId))
+                {
+                    return context.getResponseComplete();
+                }
+                else
+                {
+                    // Don't check and don't skip
+                    return false;
+                }
+            }
+            else
+            {
+                // Note if is afterPhase the return value is not relevant.
+                return context.getResponseComplete() || context.getRenderResponse();
+            }
         }
     }
 
@@ -1137,26 +1351,10 @@ public class UIViewRoot extends UIComponentBase implements UniqueIdVendor
         // throw new UnsupportedOperationException();
     }
 
-    /**
-     * DO NOT USE.
-     * <p>
-     * Although this class extends a base-class that defines a read/write id property, it makes no sense for this
-     * particular subclass to support it. The tag library does not export this property for use, but there is no way to
-     * "undeclare" a java method. Yes, this is broken OO design: direct all complaints to the JSF spec group.
-     * <p>
-     * This property should be disabled (ie throw an exception if invoked). However there are currently several places
-     * that call this method (eg during restoreState) so it just does the normal thing for the moment. TODO: fix callers
-     * then make this throw an exception.
-     *
-     * @JSFProperty tagExcluded="true"
-     */
+    @JSFProperty(tagExcluded=true)
     @Override
     public void setId(String id)
     {
-        // throw new UnsupportedOperationException();
-
-        // Leave enabled for now. Things like the TreeStructureManager call this,
-        // even though they probably should not.
         super.setId(id);
     }
     
@@ -1246,7 +1444,23 @@ public class UIViewRoot extends UIComponentBase implements UniqueIdVendor
     {
         getStateHelper().put(PropertyKeys.afterPhaseListener, afterPhaseListener);
     }
-    
+
+    /**
+     * @return the clearTransientMapOnSaveState
+     */
+    int getResetSaveStateMode()
+    {
+        return _resetSaveStateMode;
+    }
+
+    /**
+     * @param clearTransientMapOnSaveState the clearTransientMapOnSaveState to set
+     */
+    void setResetSaveStateMode(int clearTransientMapOnSaveState)
+    {
+        this._resetSaveStateMode = clearTransientMapOnSaveState;
+    }
+
     @Override
     public Map<String, Object> getAttributes()
     {
@@ -1256,6 +1470,19 @@ public class UIViewRoot extends UIComponentBase implements UniqueIdVendor
         }
         return _attributesMap;
     }
+    
+    /**
+     * @since 2.2
+     * @param context
+     * @param clientIds 
+     */
+    public void resetValues(FacesContext context,
+                        java.util.Collection<java.lang.String> clientIds)    
+    {
+        VisitContext visitContext = (VisitContext) VisitContext.createVisitContext(
+            context, clientIds, null);
+        this.visitTree(visitContext, RESET_VALUES_CALLBACK);
+    }
 
     /**
      * Indicates if the component is created when facelets builds the view and
@@ -1288,15 +1515,73 @@ public class UIViewRoot extends UIComponentBase implements UniqueIdVendor
     @Override
     public Object saveState(FacesContext facesContext)
     {
+        if (getResetSaveStateMode() == RESET_MODE_SOFT)
+        {
+            // Clear view listeners.
+            if (_systemEventListeners != null)
+            {
+                _systemEventListeners.clear();
+            }
+            if (_events != null)
+            {
+                _events.clear();
+            }
+            if (listenerSuccessMap != null)
+            {
+                listenerSuccessMap.clear();
+            }
+            _restoreViewScopeStateCalled = false;
+        }
+        if (getResetSaveStateMode() == RESET_MODE_HARD)
+        {
+            // Clear view listeners.
+            if (_systemEventListeners != null)
+            {
+                _systemEventListeners.clear();
+            }
+            if (_events != null)
+            {
+                _events.clear();
+            }
+            if (listenerSuccessMap != null)
+            {
+                listenerSuccessMap.clear();
+            }
+            if (_viewScope != null)
+            {
+                if (VIEW_SCOPE_PROXY_MAP_CLASS.isInstance(_viewScope))
+                {
+                    _viewScope = null;
+                }
+                else
+                {
+                    _viewScope.clear();
+                }
+            }
+            _restoreViewScopeStateCalled = false;
+        }
+        
         if (initialStateMarked())
         {
             Object parentSaved = super.saveState(facesContext);
+            if (_viewScope != null && 
+                Boolean.TRUE.equals(facesContext.getAttributes().get(
+                SKIP_VIEW_MAP_SAVE_STATE)))
+            {
+                if (parentSaved == null)
+                {
+                    return null;
+                }
+                return new Object[]{parentSaved, null};
+            }
+            
             if (parentSaved == null && _viewScope == null)
             {
                 //No values
                 return null;
             }
-            else if (parentSaved == null && _viewScope != null && _viewScope.size() == 0)
+            else if (parentSaved == null && _viewScope != null && _viewScope.isEmpty()
+                && !(_viewScope instanceof StateHolder) )
             {
                 //Empty view scope, no values
                 return null;
@@ -1309,6 +1594,12 @@ public class UIViewRoot extends UIComponentBase implements UniqueIdVendor
         }
         else
         {
+            if (_viewScope != null && 
+                Boolean.TRUE.equals(facesContext.getAttributes().get(
+                SKIP_VIEW_MAP_SAVE_STATE)))
+            {
+                return new Object[]{super.saveState(facesContext), null};
+            }
             Object[] values = new Object[2];
             values[0] = super.saveState(facesContext);
             values[1] = saveAttachedState(facesContext,_viewScope);
@@ -1327,7 +1618,42 @@ public class UIViewRoot extends UIComponentBase implements UniqueIdVendor
         
         Object[] values = (Object[])state;
         super.restoreState(facesContext,values[0]);
+        // JSF 2.2 spec says that restoreViewScopeState can be called but only if
+        // StateManagementStrategy is used. If that's not the case (JSF 1.2 state saving),
+        // restoreViewScopeState could not be called, so this code should avoid restore
+        // the state twice.
+        if (!_restoreViewScopeStateCalled)
+        {
+            _viewScope = (Map<String, Object>) restoreAttachedState(facesContext, values[1]);
+        }
+        else
+        {
+            _restoreViewScopeStateCalled = false;
+        }
+    }
+    
+    /**
+     * @since 2.2
+     * @param facesContext
+     * @param state 
+     */
+    public void restoreViewScopeState(FacesContext facesContext, Object state)
+    {
+        if (state == null)
+        {
+            return;
+        }
+        //StateManagementStrategy says "... obtain the state of the UIViewRoot from the 
+        // state Object returned from ResponseStateManager.getState(javax.faces.context.FacesContext,
+        // java.lang.String) and pass that to UIViewRoot.restoreViewScopeState(
+        // javax.faces.context.FacesContext, java.lang.Object).
+        // Note restoreState() will be called later, and it will restore the view. If
+        // we restore the component state here, later it could be a problem in the later
+        // restoreState() call, because the initial state will not be the same.
+        
+        Object[] values = (Object[])state;
         _viewScope = (Map<String, Object>) restoreAttachedState(facesContext, values[1]);
+        _restoreViewScopeStateCalled = true;
     }
     
     public List<SystemEventListener> getViewListenersForEventClass(Class<? extends SystemEvent> systemEvent)
@@ -1619,6 +1945,10 @@ public class UIViewRoot extends UIComponentBase implements UniqueIdVendor
 
     // we cannot make this class a inner class, because the 
     // enclosing class (UIViewRoot) would also have to be serialized.
+    /**
+     * @deprecated replaced by org.apache.myfaces.view.ViewScopeProxyMap
+     */
+    @Deprecated
     private static class ViewScope extends HashMap<String, Object>
     {
         
@@ -1644,7 +1974,7 @@ public class UIViewRoot extends UIComponentBase implements UniqueIdVendor
     /**
      * Agregates events for ANY_PHASE and current phase 
      */
-    private class Events
+    private static class Events
     {
         
         private final List<FacesEvent> _anyPhase;
@@ -1673,4 +2003,63 @@ public class UIViewRoot extends UIComponentBase implements UniqueIdVendor
             return _onPhase;
         }
     }
+    
+    private static class ResetValuesCallback implements VisitCallback
+    {
+        public VisitResult visit(VisitContext context, UIComponent target)
+        {
+            if (target instanceof EditableValueHolder)
+            {
+                ((EditableValueHolder)target).resetValue();
+            }
+            return VisitResult.ACCEPT;
+        }
+    }
+    
+    private void publishException (FacesContext facesContext, Throwable e, PhaseId phaseId, String key)
+    {
+        ExceptionQueuedEventContext context = new ExceptionQueuedEventContext (facesContext, e, null, phaseId);
+        
+        context.getAttributes().put (key, Boolean.TRUE);
+        
+        facesContext.getApplication().publishEvent (facesContext, ExceptionQueuedEvent.class, context);
+    }
+    
+    private boolean shouldViewRootPhaseListenerQueuesExceptions(FacesContext context)
+    {
+        ExternalContext ec = context.getExternalContext();
+        Boolean alwaysPerformValidationWhenRequiredTrue = (Boolean) ec.getApplicationMap().get(
+                VIEWROOT_PHASE_LISTENER_QUEUES_EXCEPTIONS_PARAM_NAME);
+
+        if (alwaysPerformValidationWhenRequiredTrue == null)
+        {
+             String param = ec.getInitParameter(VIEWROOT_PHASE_LISTENER_QUEUES_EXCEPTIONS_PARAM_NAME);
+
+             // null means the same as auto.
+             if (param == null)
+             {
+                 param = "false";
+             }
+             else
+             {
+                 // The environment variables are case insensitive.
+                 param = param.toLowerCase();
+             }
+
+             if (param.equals("true"))
+             {
+                 alwaysPerformValidationWhenRequiredTrue = true;
+             }
+             else
+             {
+                 alwaysPerformValidationWhenRequiredTrue = false;
+             }
+
+             // cache the parsed value
+             ec.getApplicationMap().put(VIEWROOT_PHASE_LISTENER_QUEUES_EXCEPTIONS_PARAM_NAME, 
+                     alwaysPerformValidationWhenRequiredTrue);
+        }
+
+        return alwaysPerformValidationWhenRequiredTrue;
+    }    
 }
