@@ -25,17 +25,26 @@ import javax.faces.FacesWrapper;
 import javax.faces.context.FacesContext;
 
 /**
- * @author Simon Lessard (latest modification by $Author: bommel $)
- * @version $Revision: 1187701 $ $Date: 2011-10-22 12:21:54 +0000 (Sat, 22 Oct 2011) $
- * 
  * @since 2.0
  */
 public abstract class VisitContextFactory implements FacesWrapper<VisitContextFactory>
 {
+    private VisitContextFactory delegate;
+
+    @Deprecated
+    public VisitContextFactory()
+    {
+    }
+
+    public VisitContextFactory(VisitContextFactory delegate)
+    {
+        this.delegate = delegate;
+    }
+    
     public abstract VisitContext getVisitContext(FacesContext context, Collection<String> ids, Set<VisitHint> hints);
 
     public VisitContextFactory getWrapped()
     {
-        return null;
+        return delegate;
     }
 }
