@@ -20,11 +20,13 @@ package javax.faces.event;
 
 /**
  * see Javadoc of <a href="http://java.sun.com/javaee/javaserverfaces/1.2/docs/api/index.html">JSF Specification</a>
- * 
- * @author Thomas Spiegl (latest modification by $Author: bommel $)
- * @version $Revision: 1187701 $ $Date: 2011-10-22 12:21:54 +0000 (Sat, 22 Oct 2011) $
  */
 public interface ActionListener extends FacesListener
 {
+    /**
+     * @since 2.2
+     */
+    static final String TO_FLOW_DOCUMENT_ID_ATTR_NAME = "to-flow-document-id";
+    
     void processAction(ActionEvent actionEvent) throws AbortProcessingException;
 }
