@@ -24,15 +24,24 @@ import java.util.Iterator;
 
 /**
  * see Javadoc of <a href="http://java.sun.com/javaee/javaserverfaces/1.2/docs/api/index.html">JSF Specification</a>
- *
- * @author Manfred Geiler (latest modification by $Author$)
- * @version $Revision$ $Date$
  */
 public abstract class RenderKitFactory implements
         FacesWrapper<RenderKitFactory>
 {
     public static final String HTML_BASIC_RENDER_KIT = "HTML_BASIC";
 
+    private RenderKitFactory delegate;
+
+    @Deprecated
+    public RenderKitFactory()
+    {
+    }
+
+    public RenderKitFactory(RenderKitFactory delegate)
+    {
+        this.delegate = delegate;
+    }
+    
     public abstract void addRenderKit(String renderKitId,
                                       RenderKit renderKit);
 
@@ -53,6 +62,6 @@ public abstract class RenderKitFactory implements
      */
     public RenderKitFactory getWrapped()
     {
-        return null;
+        return delegate;
     }
 }
