@@ -24,6 +24,7 @@ import java.util.List;
 import java.util.Map;
 
 import javax.el.MethodExpression;
+import javax.faces.application.ResourceHandler;
 import javax.faces.component.PartialStateHolder;
 import javax.faces.component.UIComponent;
 import javax.faces.component.UniqueIdVendor;
@@ -47,7 +48,6 @@ import javax.faces.view.facelets.TagHandler;
 import org.apache.myfaces.buildtools.maven2.plugin.builder.annotation.JSFFaceletAttribute;
 import org.apache.myfaces.buildtools.maven2.plugin.builder.annotation.JSFFaceletTag;
 import org.apache.myfaces.shared.renderkit.JSFAttr;
-import org.apache.myfaces.shared.renderkit.html.util.ResourceUtils;
 import org.apache.myfaces.view.facelets.AbstractFaceletContext;
 import org.apache.myfaces.view.facelets.FaceletCompositionContext;
 import org.apache.myfaces.view.facelets.tag.TagHandlerUtils;
@@ -79,8 +79,8 @@ import org.apache.myfaces.view.facelets.tag.ui.InsertHandler;
  * <li>This tag works as a attached object handler, but note on the api there is no component
  * to define a target for a behavior. See comment inside apply() method.</li>
  * </ul>
- * @author Leonardo Uribe (latest modification by $Author: lu4242 $)
- * @version $Revision: 1671154 $ $Date: 2015-04-03 20:15:44 +0000 (Fri, 03 Apr 2015) $
+ * @author Leonardo Uribe (latest modification by $Author$)
+ * @version $Revision$ $Date$
  */
 @JSFFaceletTag(name = "f:ajax")
 public class AjaxHandler extends TagHandler implements
@@ -153,6 +153,16 @@ public class AjaxHandler extends TagHandler implements
     @JSFFaceletAttribute(name = "render", className = "javax.el.ValueExpression",
                          deferredValueType = "java.lang.Object")
     private final TagAttribute _render;
+    /**
+     * 
+     */
+    @JSFFaceletAttribute(name = "delay", className = "javax.el.ValueExpression",
+                         deferredValueType = "java.lang.String")
+    private final TagAttribute _delay;
+    
+    @JSFFaceletAttribute(name = "resetValues", className = "javax.el.ValueExpression",
+            deferredValueType = "java.lang.Boolean")
+    private final TagAttribute _resetValues;
     
     private final boolean _wrapMode;
 
@@ -167,7 +177,8 @@ public class AjaxHandler extends TagHandler implements
         _onerror = getAttribute("onerror");
         _onevent = getAttribute("onevent");
         _render = getAttribute("render");
-        
+        _delay = getAttribute("delay");
+        _resetValues = getAttribute("resetValues");
         // According to the spec, this tag works in two different ways:
         // 1. Apply an ajax behavior for a selected component in this way
         //    <x:component><f:ajax ..../></x:component>
@@ -211,8 +222,7 @@ public class AjaxHandler extends TagHandler implements
             AbstractFaceletContext actx = (AbstractFaceletContext) ctx;
             // Push and pop this ajax handler to the stack, to delegate the
             // call to applyAttachedObject to ComponentTagHandlerDelegate
-            // TODO: The spec is not clear about how to deal with 
-            // composite component instances. The default one proposed here is
+            // The default one proposed here is
             // use a different stack on DefaultFaceletContext.applyCompositeComponent,
             // so components inside composite:implementation tag will not be
             // affected by f:ajax outsider handlers.
@@ -268,8 +278,8 @@ public class AjaxHandler extends TagHandler implements
         {
             UIComponent outputScript = facesContext.getApplication().
                 createComponent(facesContext, "javax.faces.Output", "javax.faces.resource.Script");
-            outputScript.getAttributes().put(JSFAttr.NAME_ATTR, ResourceUtils.JSF_JS_RESOURCE_NAME);
-            outputScript.getAttributes().put(JSFAttr.LIBRARY_ATTR, ResourceUtils.JAVAX_FACES_LIBRARY_NAME);
+            outputScript.getAttributes().put(JSFAttr.NAME_ATTR, ResourceHandler.JSF_SCRIPT_RESOURCE_NAME);
+            outputScript.getAttributes().put(JSFAttr.LIBRARY_ATTR, ResourceHandler.JSF_SCRIPT_LIBRARY_NAME);
             outputScript.getAttributes().put(JSFAttr.TARGET_ATTR, "head");
 
             //AbstractFaceletContext actx = (AbstractFaceletContext) ctx;
@@ -472,7 +482,31 @@ public class AjaxHandler extends TagHandler implements
             ajaxBehavior.setValueExpression("render", _render
                     .getValueExpression(faceletContext, Object.class));
         }
-        
+        if (_delay != null)
+        {
+            if (_delay.isLiteral())
+            {
+                ajaxBehavior.setDelay(_delay.getValue(faceletContext));
+            }
+            else
+            {
+                ajaxBehavior.setValueExpression("delay", _delay
+                    .getValueExpression(faceletContext, String.class));
+            }
+        }
+       if (_resetValues != null)
+        {
+            if (_resetValues.isLiteral())
+            {
+                ajaxBehavior
+                        .setResetValues(_resetValues.getBoolean(faceletContext));
+            }
+            else
+            {
+                ajaxBehavior.setValueExpression("resetValues", _resetValues
+                        .getValueExpression(faceletContext, Boolean.class));
+            }
+        }
         cvh.addClientBehavior(eventName, ajaxBehavior);
     }
 
